from datetime import datetime

import numpy as np


####
### %Y : yyyy
### %m : mm
### %d : dd
def date_format(dt: datetime, pattern="%Y%m%d"):
    return dt.strftime(pattern)


def datestr_format(dt: str, from_pattern, to_pattern="%Y-%m-%d"):
    try:
        return datetime.strptime(dt, from_pattern).strftime(to_pattern)
    except Exception as ex:
        return None

def get_quarter_start_date(quarter):
    month = int(quarter) % 100
    month -= 2

    return f"{month:02d}.01"


def get_last_quarter(year_month, step=1):
    year_month = int(year_month)
    year_month -= 1
    quarters = [3, 6, 9, 12]
    while year_month % 100 not in quarters:
        year_month -= 1
        if year_month % 100 == 0:
            year_month -= 100
            year_month += 12
    if step == 1:
        return year_month
    else:
        return get_last_quarter(year_month, step=step-1)


def diff_days(start: str, end: str, format="%Y-%m-%d") -> int:
    """
    두 날짜 사이의 일수를 구한다.
    :param start:
    :param end:
    :param format:
    :return:
    """
    if start is None or end is None or start.strip() == "" or end.strip() == "":
        return np.nan

    try:
        days = (datetime.strptime(end, format) - datetime.strptime(start, format)).days
        return days
    except Exception as ex:
        return np.nan


def time_in_range(start, end, dt):
    fmt = "%Y-%m-%d_%H:%M"
    if start <= end:
        return start <= dt <= end
    else:
        print(f"dt [{dt.strftime(fmt)}] is not in range(start : [{start.strftime(fmt)}], end : [{end.strftime(fmt)}]")
        return start <= dt or dt <= end


if __name__ == "__main__":
    print(f"get_last_quarter(202207) : {get_last_quarter('202207')}")
    print(f"get_last_quarter(202206) : {get_last_quarter(202206)}")
    print(f"get_last_quarter(202205) : {get_last_quarter(202205)}")


import unittest

import numpy as np

from crawlutils.text_utils import fmt_int, fmt_float


class TestTextUtils(unittest.TestCase):

    def test_fmt_int(self):
        self.assertEqual(fmt_int(1000, use_comma=True, default="-", postfix="원"), "1,000원")
        self.assertEqual(fmt_int(1000, use_comma=False, default="-", postfix="원"), "1000원")
        self.assertEqual(fmt_int(1000, use_comma=False, default="-", postfix=""), "1000")
        self.assertEqual(fmt_int(np.nan, use_comma=True, default="-", postfix="원"), "-")
        self.assertEqual(fmt_int("-", use_comma=True, default="-", postfix="원"), "-")
        self.assertEqual(fmt_int("-", use_comma=True, default="*", postfix="원"), "*")

    def test_fmt_float(self):
        self.assertEqual(fmt_float(1000, use_comma=True, precision=2, default="-", postfix="원"), "1,000.00원")
        self.assertEqual(fmt_float(1000, use_comma=False, precision=0, default="-", postfix="원"), "1000원")
        self.assertEqual(fmt_float(1000.12, use_comma=False, precision=1, default="-", postfix=""), "1000.1")
        self.assertEqual(fmt_float(np.nan, use_comma=True, default="-", postfix="원"), "-")
        self.assertEqual(fmt_float("-", use_comma=True, default="-", postfix="원"), "-")
        self.assertEqual(fmt_float("-", use_comma=True, default="*", postfix="원"), "*")


import re
import numpy as np


def make_text(t) -> str:
    return t.replace('\n', '').replace('\t', '').replace(' ', '').replace('\r', '').replace('\xa0', '')


def make_num(t) -> str:
    return t.replace('\n', '').replace('\t', '').replace(' ', '').replace('\r', '').replace('\xa0', '').replace(',', '')


def make_num_include_zo(txt):
    """
    ## 억원 이 생략된 억단위 금액을 숫자로 변환
    ## 1조 2,000 => 1_200_000_000_000
    :param txt:
    :return:
    """
    일조 = 1_000_000_000_000
    일억 = 100_000_000
    tkns = txt.split("조")
    zo_part = int(make_num(tkns[0])) * 일조 if len(tkns) > 1 else 0
    uk_part_txt = tkns[1] if len(tkns) > 1 else make_num(tkns[0])
    uk_part = int(make_num(uk_part_txt)) * 일억
    return zo_part + uk_part


def make_date(data):
    return data.replace('년', '-').replace('월', '-').replace('일', '').replace(' ', '').replace('.', '-')


def to_quarter_str(num):
    return f"{num // 100}.{(num % 100) // 3}"


def splitter(s):
    spl = s.split(" = ")
    if len(spl) <= 1:
        return None
    try:
        return int(spl[1])
    except ValueError:
        return None


def has_text_in_list(texts, str):
    for text in texts:
        if text in str:
            return True

    return False


def remove_strs(text, removes):
    tmp = text
    for remove in removes:
        tmp = tmp.replace(remove, '')

    return tmp


def trim(text):
    return text.strip() if text is not None else ""


def fmt_int(num, use_comma=True, default: str = "-", postfix: str = "") -> str:
    if num is None:
        return default
    elif isinstance(num, str):
        t = remove_strs(num, [",", " ", "원"])
        if t == "-":
            return default
        num = int(t) if t != "" else np.nan

    if num != num: # isnan
        return default
    else:
        return f"{int(num):,}{postfix}" if use_comma else f"{int(num)}{postfix}"


def fmt_float(num, use_comma=True, precision: int = 2, default: str = "-", postfix: str = "") -> str:
    if num is None:
        return default
    elif isinstance(num, str):
        t = remove_strs(num, [",", " ", "원"])
        if t == "-":
            return default
        num = float(t) if t != "" else np.nan
    elif isinstance(num, int):
        num = float(num)
    if num != num: # isnan
        return default
    else:
        return f"{num:,.{precision}f}{postfix}" if use_comma else f"{num:.{precision}f}{postfix}"


def default_str(text, default: str = "-") -> str:
    return text if text is not None else default


def trim_template(text):
    return re.sub(r"^\s*\|", "", text.strip(), 0, re.MULTILINE) + "\n\n"


def json_encode(dictionary):
    return dictionary.encode()


def escape_for_markdown(msg: str):
    if msg is None:
        return ""

    return msg.replace("(", "\(").replace(")", "\)")\
        .replace(".", "\.").replace("-", "\-")\
        .replace("=", "\=").replace("*", "\*")


if __name__ == "__main__":
    r = fmt_float(124.2356, use_comma=True, precision=2, default="-", postfix="원")
    print(r)


import unittest
import re

from bs4 import BeautifulSoup

from crawlutils.soup_traversal import get_td_text, get_td_value_after, get_td, get_td_regex, get_td_text_regex, \
    find_table, get_td_value


class TestSoupTraversal(unittest.TestCase):

    def test_get_td_regex(self):
        html = """
            <table>
                <tr><td>정정사유</td><td>정정사유값1</td></tr>
                <tr><td>정정사2</td><td>정정사유값2</td></tr>
                <tr><td>정정사3</td><td>정정사유값3</td></tr>
                <tr><td>정정사4</td><td>정정사유값4</td></tr>
            </table> 
        """
        soup = BeautifulSoup(html, "html.parser")
        r = get_td_regex(soup.select("td"), r'정.*3')
        self.assertEqual('정정사3', r.text)

    def test_get_td_text_regex(self):
        html = """
            <table>
                <tr><td>정정사유</td><td>정정사유값1</td></tr>
                <tr><td>정정사2</td><td>정정사유값2</td></tr>
                <tr><td>정정사3</td><td>정정사유값3</td></tr>
                <tr><td>정정사4</td><td>정정사유값4</td></tr>
            </table> 
        """
        soup = BeautifulSoup(html, "html.parser")
        r = get_td_text_regex(soup.select("td"), r'정.*3')
        self.assertEqual('정정사유값3', r)

    def test_soup_traversal(self):
        html = """
            <table>
                <tr><td>정정사유</td><td>정정사유값</td></tr>
            </table>
        """
        soup = BeautifulSoup(html, "html.parser")
        r = get_td_value(soup.select("td"), '정정사유')
        self.assertEqual(r, '정정사유값')


    def test_soup_traversal2(self):
        html = """
            <table>
                <tr><td>정정사유</td><td>정정사유값1</td></tr>
                <tr><td>정정사2</td><td>정정사유값2</td></tr>
                <tr><td>정정사3</td><td>정정사유값3</td></tr>
                <tr><td>정정사4</td><td>정정사유값4</td></tr>
            </table>
        """
        soup = BeautifulSoup(html, "html.parser")
        r = get_td_value(soup.select("td"), '정정사1', '정정사3')
        self.assertEqual('정정사유값3', r)

    def test_get_td_text_after(self):
        html = """
        <table>
          <tr> 
       <td width="124"> <span style="width:124px;font-size:10pt;">계약금액 총액(원)</span> </td> 
       <td colspan="2" width="332"> <span class="xforms_input" style="width:332px;font-size:10pt;text-align:right;">11,380,800,000</span> </td> 
      </tr> 
      <tr> 
       <td width="124"> <span style="width:124px;font-size:10pt;">최근 매출액(원)</span> </td> 
       <td colspan="2" width="332"> <span class="xforms_input" style="width:332px;font-size:10pt;text-align:right;">70,440,395,777</span> </td> 
      </tr> 
      <tr> 
       <td width="124"> <span style="width:124px;font-size:10pt;">매출액 대비(%)</span> </td> 
       <td colspan="2" width="332"> <span class="xforms_input" style="width:332px;font-size:10pt;text-align:right;">16.16</span> </td> 
      </tr> 
      <tr> 
       <td colspan="2" width="268"> <span style="width:268px;font-size:10pt;">3. 계약상대방</span> </td> 
       <td colspan="2" width="332"> <span class="xforms_input" style="width:332px;font-size:10pt;">-</span> </td> 
      </tr> 
      <tr> 
       <td colspan="2" width="268"> <span style="width:268px;font-size:10pt;">-최근 매출액(원)</span> </td> 
       <td colspan="2" width="332"> <span class="xforms_input" style="width:332px;font-size:10pt;text-align:right;">1,238,811</span> </td> 
      </tr> 
      <tr> 
       <td colspan="2" width="268"> <span style="width:268px;font-size:10pt;">-주요사업</span> </td> 
       <td colspan="2" width="332"> <span class="xforms_input" style="width:332px;font-size:10pt;">-</span> </td> 
      </tr> 
      <tr> 
       <td colspan="2" width="268"> <span style="width:268px;font-size:10pt;">-회사와의 관계</span> </td> 
       <td colspan="2" width="332"> <span class="xforms_input" style="width:332px;font-size:10pt;">-</span> </td> 
      </tr> 
      <tr> 
      </table>
        """
        soup = BeautifulSoup(html, "html.parser")
        tds = soup.select("td")
        anchor_td = get_td(tds, "계약상대방")
        r = get_td_value_after(anchor_td, 5, "최근매출액")
        self.assertEqual("1,238,811", r)

    def test_find_table(self):
        html = """
            <table>
                <tr><td>계약내용</td><td>정정사유값</td></tr>
                <tr><td>정정전</td><td>정정사유값</td></tr>
            </table>
            <table>
                <tr><td>계약내용</td><td>125125</td></tr>
            </table>
            <table>
                <tr><td>정 정사</td><td>정정사이값</td></tr>
            </table>
            <table>
                <tr><td>정 정사</td><td>정정사이값</td></tr>
            </table>
        """
        soup = BeautifulSoup(html, "html.parser")
        r = find_table(soup.findAll("table"), contains_any=['계약내용',"기한"], not_contains_any=['정정전', '정정후'])
        self.assertEqual(r, soup.select("table")[1])


import unittest

import numpy as np

from crawlutils.num_utils import to_int, to_float


class TestNumUtils(unittest.TestCase):

    def test_to_int(self):
        self.assertEqual(to_int("+1,000"), 1000)
        self.assertEqual(to_int("1,000"), 1000)
        self.assertEqual(to_int("-1,000"), -1000)
        self.assertEqual(to_int("1,000.24"), 1000)
        self.assertEqual(to_int("-1,000.24"), -1000)
        self.assertTrue(np.isnan(to_int(np.nan)))
        self.assertTrue(np.isnan(to_int(" ")))
        self.assertTrue(np.isnan(to_int("-")))
        self.assertTrue(np.isnan(to_int(" - ")))

    def test_to_float(self):
        self.assertEqual(to_float("1,000"), 1000)
        self.assertEqual(to_float("-1,000"), -1000)
        self.assertEqual(to_float("1,000.24"), 1000.24)
        self.assertEqual(to_float("-1,000.24"), -1000.24)
        self.assertTrue(np.isnan(to_float(np.nan)))
        self.assertTrue(np.isnan(to_float(" ")))
        self.assertTrue(np.isnan(to_float("-")))
        self.assertTrue(np.isnan(to_float(" - ")))


import re
from typing import List, Tuple
from bs4 import PageElement
from crawlutils.num_utils import defaultIfNumber
from crawlutils.text_utils import remove_strs


def get_td(td_list: List[PageElement], *text_in) -> PageElement:
    """
    해당 keyword 를 포함하는 td 를 찾아서 반환한다.
    :param td_list:
    :param text_in:
    :return:
    """
    for td in td_list:
        for text in text_in:
            if text in td.text.replace(" ", ""):
                return td
    return None

def get_td_regex(td_list: List[PageElement], regex) -> PageElement:
    """
    해당 keyword 를 포함하는 td 를 찾아서 반환한다.
    :param td_list:
    :param text_in:
    :return:
    """
    compiled_regex = re.compile(regex)
    for td in td_list:
        if compiled_regex.search(td.text.replace(" ", "")):
            return td
    return None


def get_td_text(td_list: List[PageElement], *text_in) -> str:
    """
    해당 td 를 찾아서 그 다음 td 의 text 를 가져온다.
    :param td_list:
    :param text_in:
    :return:
    """
    r = get_td(td_list, *text_in)
    return r.findNext('td').text.strip() if r is not None else r


def get_td_value(td_list: List[PageElement], *text_in) -> str:
    """
    해당 td 를 찾아서 그 다음 td 의 text 를 가져온다.
    :param td_list:
    :param text_in:
    :return:
    """
    r = get_td(td_list, *text_in)
    return r.findNext('td').text.strip() if r is not None else r


def get_td_text_regex(td_list: List[PageElement], regex) -> str:
    """
    해당 td 를 찾아서 그 다음 td 의 text 를 가져온다.
    :param td_list:
    :param text_in:
    :return:
    """
    r = get_td_regex(td_list, regex)
    return r.findNext('td').text.strip() if r is not None else r

def get_td_value_after(td_anchor: PageElement, limit=5, *keyword) -> str:
    """
    :param td_anchor: 기준이 되는 td
    :param limit: anchor 부모 tr 로 부터 몇개의 tr 을 가져올 것인지
    :param keyword: 찾으려는 키워드, 가변키워드, 우선순위가 높은 키워드 부터
    :return:
    """
    if td_anchor is None:
        return None
    trs: List[PageElement] = td_anchor.parent.findNextSiblings('tr')

    if len(trs) < 1:
        return None

    trs = trs[:limit]
    tds = []
    for tr in trs:
        tds.extend(tr.findAll('td'))

    return get_td_value(tds, *keyword)


def find_tr_has_text(text, trlist) -> PageElement:
    target_tr = None
    for tr in trlist:
        if text in tr.text.replace(' ', ''):
            target_tr = tr
            break
    return target_tr


def find_table_has_text(table_list: List[PageElement], *keywords) -> PageElement:
    for table in table_list:
        for keyword in keywords:
            if keyword in table.text.replace(' ', ''):
                return table
    return None


def find_table(tables: List[PageElement], contains_any: List[str], not_contains_any: List[str] = None, contains_all: List[str] = None, not_contains_all: List[str] = None):
    for table in tables:
        table_text = remove_strs(table.text, [" ", "\n", "\t", "\xa0"])
        if contains_any and not any([text in table_text for text in contains_any]):
            continue
        if not_contains_any and any([text in table_text for text in not_contains_any]):
            continue
        if contains_all and not all([text in table_text for text in contains_all]):
            continue
        if not_contains_all and all([text in table_text for text in not_contains_all]):
            continue
        return table
    return None


def extract_correction(tables: List[PageElement]) -> Tuple[str, str]:
    """
    정정관련 정보 추출
    :param tables:
    :return: Tuple(corr_reason, corr_date)
    """
    mod_table = find_table(tables, contains_any=["정정사유", "정정전"])
    if mod_table is not None:
        mod_td_list = mod_table.findAll('td')
        correction_reason = get_td_value(mod_td_list, '정정사유')
        correction_date = get_td_value(mod_td_list, '공시서류제출일')
        return correction_reason, correction_date
    return None, None


def defaultNumberInTR(text, tr3, unit_multiplier, default="-"):
    np_tr = find_tr_has_text(text, tr3)
    np = defaultIfNumber(np_tr.findAll('td')[1].text, unit_multiplier, default=default)
    return np